import argparse
import os
import hashlib
from importlib.metadata import version
from pathlib import Path
import re
from textwrap import dedent


from loguru import logger
import requests


from ytpodgen.downloader import Downloader
from ytpodgen.feedgenerator import FeedGenerator
from ytpodgen.uploader import Uploader


SLACK_WEBHOOK_URL = os.environ.get("SLACK_WEBHOOK_URL")


def send_slack_notification(message):
    data = {"text": message}
    requests.post(SLACK_WEBHOOK_URL, json=data)


def get_version():
    return version("ytpodgen")


def parse_arguments():
    parser = argparse.ArgumentParser(description="Podcast generator")
    parser.add_argument(
        "--title",
        required=True,
        help="title for the podcast"
    )
    parser.add_argument(
        "--hostname",
        required=True,
        help="hostname(custom or r2.dev) to serve files"
    )
    parser.add_argument(
        "--liveurl",
        type=validate_url,
        help="Watch for and download the specified YouTube Live URL."
    )
    parser.add_argument(
        "--upload-r2",
        action="store_true",
        help="If specified, upload mp3s/RSS to Cloudflare R2.",
    )
    parser.add_argument(
        "--output",
        default=".",
        help="Output directory(default: current directory)"
    )
    parser.add_argument(
        "--bucket",
        default="podcast",
        help="R2 Bucket name to upload files(default: podcast)"
    )
    parser.add_argument(
        "--private",
        action="store_true",
        help="If specified, make the podcast private by generating hashed url."
    )
    parser.add_argument(
        "--version",
        action="version",
        version=get_version(),
        help="Show version and exit.",
    )

    args = parser.parse_args()
    return args


def validate_url(url):
    if re.match(r"^https?://(?:www\.)?youtube\.com/watch\?v=([a-zA-Z0-9_-]{11})", url):
        return url
    if not url.startswith("https://www.youtube.com/"):
        raise argparse.ArgumentTypeError(
            f"""
            Invalid url {url}.
            It must be a YouTube Live URL that starts with https://www.youtube.com/.
            """
        )
    if not url.endswith("/live"):
        raise argparse.ArgumentTypeError(
            f"""
            Invalid url {url}.
            It must be a YouTube Live URL that ends with /live.
            """
        )
    return url


def main():
    args = parse_arguments()

    change_work_dir(args.output, args.title)
    create_logger(args.title)

    try:
        while True:
            run(args)
            if args.liveurl is None:
                break
    except Exception as e:
        logger.error(f"ytpodgen failed with following error messages: {e}")


def create_logger(title):
    logger.add(f"{title}.log", level="INFO")
    if SLACK_WEBHOOK_URL:
        logger.add(send_slack_notification, level="SUCCESS")


def change_work_dir(output, title):
    try:
        work_dir = Path.cwd()
        if output:
            work_dir = Path(output).absolute()
        os.chdir(work_dir)
    except FileNotFoundError:
        create_logger(title)
        logger.error(f"Invalid argument {output} for --output option.")
        exit()


def run(args):
    path = generate_hashed_path(args.title) if args.private else args.title

    if args.liveurl:
        logger.info("Running yt-dlp...")
        Downloader.download(args.title, args.liveurl)

    logger.info("Generating feeds...")
    FeedGenerator.generate_rss(args.title, args.hostname, path)

    if args.upload_r2:
        logger.info("Uploading files...")
        Uploader.upload_to_r2(path, args.bucket)
        logger.success(
            dedent(
                f"""
            {args.title} Uploaded.
            Podcast feed url: https://{args.hostname}/{path}/index.rss
            """
            ).strip("\n")
        )


def generate_hashed_path(title):
    hashed_path = hashlib.sha256(title.encode("utf-8")).hexdigest()
    return hashed_path


if __name__ == "__main__":
    main()



import os
from pathlib import Path
import re


from botocore.exceptions import ClientError
import boto3
from loguru import logger


class Uploader:
    def __init__(self):
        pass

    @staticmethod
    def upload_to_r2(path, bucket="podcast"):
        s3 = boto3.resource(
            "s3",
            endpoint_url=Uploader.get_env_var("R2_ENDPOINT_URL"),
            aws_access_key_id=Uploader.get_env_var("R2_ACCESS_KEY_ID"),
            aws_secret_access_key=Uploader.get_env_var("R2_SECRET_ACCESS_KEY"),
            region_name="auto"
        )

        filetypes = r"(.+\.mp3|index.rss)"  # regex pattern in string
        files = Uploader.collect_files(filetypes)
        podcastbucket = Uploader.get_or_create_bucket(bucket, s3)
        Uploader.upload(path, files, podcastbucket)

    @staticmethod
    def get_env_var(name):
        try:
            envvar = os.environ[name]
            return envvar
        except KeyError:
            raise KeyError(f"{name} not set.")

    @staticmethod
    def get_or_create_bucket(bucket, s3):
        podcastbucket = s3.Bucket(bucket)
        if not Uploader.bucket_exists(podcastbucket, s3):
            podcastbucket.create()
        return podcastbucket

    @staticmethod
    def collect_files(pattern):
        files = [p for p in Path(".").glob("*") if re.search(pattern, str(p))]
        if not len(files):
            raise FileNotFoundError("required files not found.")
        return files

    @staticmethod
    def upload(path, upload_files, podcastbucket):
        for file in upload_files:
            try:
                podcastbucket.upload_file(file, f"{path}/{file}")
            except ClientError as e:
                logger.error(e)

    @staticmethod
    def bucket_exists(s3bucket, s3) -> bool:
        return s3bucket in s3.buckets.all()



import datetime
from pathlib import Path


from podgen import Podcast, Episode, Media


class FeedGenerator:
    def __init__(self):
        pass

    @staticmethod
    def generate_rss(title, hostname, path):
        podcast = Podcast(
            name=title,
            website=f"https://{hostname}/{path}/",
            description=title,
            explicit=False,
            withhold_from_itunes=True,
        )

        FeedGenerator._add_episodes(hostname, podcast, path)
        podcast.rss_file("index.rss")

    @staticmethod
    def _add_episodes(hostname, podcast, path):
        files = Path(".").glob("*.mp3")
        for file in sorted(files, reverse=True):
            media = Media(
                url=f"https://{hostname}/{path}/{file.name}",
                size=file.stat().st_size,
            )
            media.populate_duration_from(file)

            podcast.add_episode(
                Episode(
                    title=file.name,  # filename
                    media=media,
                    publication_date=datetime.datetime.fromtimestamp(
                        file.stat().st_mtime,
                        datetime.timezone.utc,
                    ),
                )
            )



import yt_dlp


class Downloader:
    def __init__(self):
        pass

    @staticmethod
    def download(title, liveurl):
        live_from_start = False
        
        # See help(yt_dlp.YoutubeDL) for a list of available options
        # and public functions
        # https://github.com/yt-dlp/yt-dlp/blob/216bcb66d7dce0762767d751dad10650cb57da9d/yt_dlp/YoutubeDL.py#L184
        ydl_opts = {
            # best audio-only format if available,
            # and if not, fall back to best format that contains both video and audio
            "format": "bestaudio/best",
            "wait_for_video": (60, 600),
            "outtmpl": f"{title}_%(epoch>%Y%m%d%H%M%S)s_%(id)s",
            "live_from_start": live_from_start,
            "postprocessors": [
                {  # Extract audio using ffmpeg
                    "key": "FFmpegExtractAudio",
                    "preferredcodec": "mp3",
                    "preferredquality": "64",
                }
            ],
            "postprocessor_args": [
                "-ac",
                "1",  # audio channel mono
            ],
        }

        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            ydl.download(liveurl)
